#ifdef WIN32
#include <windows.h>
#endif

#include <assert.h>
#include <iostream.h>
#include <GL/gl.h>

#include "Layer.h"
#include "Rect.h"
#include "Opt.h"

#ifdef DEBUG
	// #define DRAW_BB
#endif

// aby nie dolaczac calej biblioteki FOX 
// skopiowano tutaj tylko makra i definicje typow
// *************************************************************************************************
typedef unsigned char FXuchar;
typedef unsigned int FXuint;

// Make RGBA color
#define FXRGBA(r,g,b,a) (((FXuint)(FXuchar)(r)) | ((FXuint)(FXuchar)(g)<<8) | ((FXuint)(FXuchar)(b)<<16) | ((FXuint)(FXuchar)(a)<<24))
// Get alpha value from RGBA color
#define FXALPHAVAL(rgba)   ((FXuchar)((FXuint)(rgba)>>24))
// Get blue value from RGBA color
#define FXBLUEVAL(rgba) ((FXuchar)((FXuint)(rgba)>>16))
// Get green value from RGBA color
#define FXGREENVAL(rgba)  ((FXuchar)((FXuint)(rgba)>>8))
// Get red value from RGBA color
#define FXREDVAL(rgba) ((FXuchar)(rgba))
// *************************************************************************************************


// TODO: ustawienie odpowiednich wzorow dla "GL_POLYGON_STIPPLE" (32x32 bity)

static GLubyte halftone[] = 
{
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
	0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55 
};

// ================================================================== int GetIdByName(const char* s)
//
int GetIdByName(const char* s)
{
	assert(s!=NULL);

	if(s[0]=='O' && s[1]=='P') return LR_OPTIONS;
	
	if(s[0]=='M') 
		switch(s[1])
		{
			case '6': return LR_METAL6;
	 		case '5': return LR_METAL5;
			case '4': return LR_METAL4;
			case '3': return LR_METAL3;
			case '2': return LR_METAL2; 
			case 'E': return LR_METAL1;
		}
	
	if(s[0]=='C' && s[1]=='O') return LR_CONTACT;
	if(s[0]=='P' && s[1]=='A') return LR_LOCAL; // FIXME: jaka toto ma nazwe ? LC ?
	if(s[0]=='P' && s[1]=='O') return LR_POLY;
	if(s[0]=='D' && s[1]=='P') return LR_DIFFP;
	if(s[0]=='D' && s[1]=='N') return LR_DIFFN;
	if(s[0]=='N' && s[1]=='W') return LR_NWELL;
	
// FIXME: dorobic obsluge warstwy VIA
	if(s[0]=='V' && s[1]=='I') return LR_CONTACT;
	
	return LR_NULL;
}

// ======================================================================= char* GetNameById(int id)
//
char* GetNameById(int id)
{
	switch(id)
	{
		case LR_NWELL   : return "NW";
		case LR_DIFFN   : return "DN";
		case LR_DIFFP   : return "DP";
		case LR_POLY    : return "PO";
		case LR_METAL1  : return "ME";
		case LR_METAL2  : return "M2";
		case LR_METAL3  : return "M3";
		case LR_METAL4  : return "M4";
		case LR_METAL5  : return "M5";
		case LR_METAL6  : return "M6";
		case LR_CONTACT : return "CO";
		case LR_OPTIONS : return "OP";
		case LR_LOCAL   : return "LC"; // FIXME: jaka nazwe ma LOCAL?
		case LR_NULL    : return "NULL!";
	}	

	return "UNKNOWN";
}

// =================================================================== char* GetStippleById(int nid)
//
GLubyte* GetStippleById(int nid)
{
	switch(nid)
	{
		case LR_NWELL   : return halftone;
		case LR_DIFFN   : return halftone;
		case LR_DIFFP   : return halftone;
		case LR_POLY    : return halftone;
		case LR_METAL1  : return halftone;
		case LR_METAL2  : return halftone;
		case LR_METAL3  : return halftone;
		case LR_METAL4  : return halftone;
		case LR_METAL5  : return halftone;
		case LR_METAL6  : return halftone;
		case LR_CONTACT : return halftone;
		case LR_OPTIONS : return halftone;
		case LR_LOCAL   : return halftone; // FIXME: jaka nazwe ma LOCAL?
		case LR_NULL    : return halftone;
	}

	return halftone;
}

// =========================================== GroupNode* Layer::GetNext(int what, void* ptr = NULL)
//
GroupNode* Layer::GotoNext(int what, void* ptr)
{
	assert(head!=NULL);
	active = (active==NULL) ? head->GetNext() :  active->GetNext();

	if(what==LT_ANY)
		return active;

	while(active!=NULL)
	{
		switch(what)
		{
			case LT_SELECTED: 
				if(active->Selected()) return active; 
				break;
				
			case LT_NOTSELECTED: 
				if(!active->Selected()) return active; 
				break;
					
			case LT_RECTOVER: case LT_NOTRECTOVER:
			{
				assert(ptr!=NULL);
				Rect* r = (Rect*) ptr;
				if((what==LT_RECTOVER) ? r->InMe(*active) : !r->InMe(*active))
					return active;
				break;
			}
			
			case LT_RECTXYIN: case LT_NOTRECTXYIN:
			{
				assert(ptr!=NULL);
				Rect* r = (Rect*) ptr;
				if((what==LT_RECTXYIN) ? active->InMe(r->X(),r->Y()) : !active->InMe(r->X(),r->Y()))
					return active;
				break;
			}
		} // switch(what)
		active = active->GetNext();
	} // while(active!=NULL)

	return active; // active==NULL zawsze w tym miejcu
}

// ======================================================================== void Layer::Rewind(void)
//
void Layer::Rewind() { active=NULL; }

// ============================================================================== Layer::Layer(void) 
//
Layer::Layer(void) : Rect()
{
	color=id=enabled=0;
	active = head = new GroupNode();
}

// ============================================================================= Layer::~Layer(void)
//
Layer::~Layer(void) { delete head; }

// ==================================================================== int Layer::GetId(void) const
//
int Layer::GetId(void) const { return id; }

// ====================================================================== void Layer::SetId(int nid)
//
void Layer::SetId(int nid) { id=nid; }

// ================================================================= int Layer::GetColor(void) const
//
int Layer::GetColor(void) const { return color; }

// ==================================================================== void Layer::SetColor(int nc)
//
void Layer::SetColor(int nc) { color = nc; }

// ======================================================================== void Layer::Enable(void)
//
void Layer::Enable(void) { enabled = 1; }

// ======================================================================= void Layer::Disable(void)
//
void Layer::Disable(void) { enabled = 0; }

// ==================================================================== void Layer::InvEnabled(void) 
//
void Layer::InvEnabled(void) { enabled = !enabled; }

// ================================================================== int Layer::Enabled(void) const
//
int Layer::Enabled(void) const { return enabled; }

// =========================== int Layer::Action(int what, int target = LT_ANY, void* helper = NULL)
//
int Layer::Action(int what, int target, void* helper)
{
	if(!enabled) // tylko wlaczone warstwy wykonuja polecenia
		return 0;
		
	int count = 0;

	if(what == LA_ADD)  // dodawac mozna tylko elementy dzidziczace z GroupNode!
	{
		assert(head!=NULL);
		assert(helper!=NULL);
		if(helper!=NULL) Grow(*((GroupNode*) helper));
		head->Add((GroupNode*) helper);
		count++;
	}
	else if(what == LA_DELETE)
	{
		GroupNode* t = head;
		Rewind();
		GotoNext(target,helper);
		while(t!=NULL)
			if((t->GetNext()==active) && (active!=NULL))
			{
				GotoNext(target,helper);
				t->DeleteNext();
				count++;
				if(t->GetNext() == NULL) 
					t = NULL;
			}
			else
				t = t->GetNext();

		ChangeTo(0,0,0,0);
		Action(LA_GROW,LT_ANY);
	}
	else
	{
		Rewind();
		GroupNode* t = GotoNext(target,helper);
		while(t!=NULL)
		{
			switch(what)
			{
				case LA_SELECT:    t->Select();    count++;  break;
				case LA_DESELECT:  t->Deselect();  count++;  break;
				case LA_INVSELECT: t->InvSelect(); count++;  break;
				case LA_GROW:      Grow(*t);       count++;  break;
			}
			t = GotoNext(target,helper);
		}
	} //  if(what == LA_DELETE) { } else { }
	return count;
}

// ============================================================ void Layer::Draw(int mode = 0) const
//
void Layer::Draw(int) const
{
	assert(head!=NULL);
	GroupNode* t = head->GetNext();
	Rect sel;

	glPushAttrib(GL_LINE_BIT|GL_ENABLE_BIT|GL_POLYGON_STIPPLE_BIT|GL_POLYGON_BIT);
		glColor4ub(FXREDVAL(color), FXGREENVAL(color), FXBLUEVAL(color), FXALPHAVAL(color));
		// glEnable(GL_POLYGON_STIPPLE);
		// glPolygonStipple(GetStippleById(id));
		
		glPolygonMode(GL_FRONT_AND_BACK, enabled ? GL_FILL : GL_LINE); 
		while(t != NULL)
		{
			t->Draw();
			t = t->GetNext();
		}

		if(enabled) // rysowanie zaznaczonych elementow - tylko gdy Enabled()
		{
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
			glEnable(GL_LINE_STIPPLE);
			glColor4ub(255,255,255,255);
			glPushMatrix();
				glTranslatef(0.0,0.0,0.01);
	
				t = head->GetNext();
				while(t != NULL)
				{
					if(t->Selected()) t->Draw();
					t = t->GetNext();
				}
			glPopMatrix();

			#ifdef DRAW_BB
				glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
				glDisable(GL_LINE_STIPPLE);
				glColor4ub(99,99,99,255);
				glPushMatrix();
					glTranslatef(0.0,0.0,0.01);
					glBegin(GL_QUADS);
						glVertex2f(x,y);
						glVertex2f(x+width,y);
						glVertex2f(x+width,y+height);
						glVertex2f(x,y+height);
					glEnd();
				glPopMatrix();
			#endif
		}

	glPopAttrib();
}

// ============================================================= int Layer::Save(ostream& out) const 
//
int Layer::Save(ostream& out) const
{
	assert(head!=NULL);
	GroupNode* t = head->GetNext();
	char* name = GetNameById(id);
	Rect r;

	while(t != NULL)
	{
		if(out.fail()) return 0; // FIXME: obsluga bledu (messagebox)
		r.ChangeTo(*t);

		out<<"REC("<<r.X(); // TODO: zapisywanie obiektow innego typu (TEXT, $?, SIMUL...)
		out<<","<<r.Y();
		out<<","<<r.Width();
		out<<","<<r.Height();
		out<<","<<name;
		if(id==LR_OPTIONS)
		{
			Opt* o = (Opt*) t;
			out<<","<<o->GetData();
		}
		out<<")"<<endl;

		t = t->GetNext();
	}	

	return 1;
}
