#ifdef WIN32
#include <windows.h>
#endif
#include <iostream.h>
#include <GL/gl.h>
#include "MouseRect.h"

// ============================================================= MouseRect::MouseRect(void) : Rect()
//
MouseRect::MouseRect(void) : Rect()
{
	left=right=mousex=mousey=clickx=clicky=0;
	mode=MM_NONE;
}

// ===================================================================== MouseRect::~MouseRect(void)
//
MouseRect::~MouseRect(void) { }

// ======================================================== void MouseRect::LeftDown(int nx, int ny)
//
void MouseRect::LeftDown(int nx, int ny)
{
	if((right==0) && (left==0)) 
	{ 
		ChangeTo(x=clickx=nx,y=clicky=ny,0,0); 
		left=1; 
	}
	else left=-1;
	
	mousex=nx; mousey=ny;
}

// ========================================================== void MouseRect::LeftUp(int nx, int ny)
//
void MouseRect::LeftUp(int nx, int ny)
{
	if(left==1) 
	{ 
		ChangeTo(clickx,clicky,nx-clickx,ny-clicky);
		left=0; 
	}
	else left=0;
	mousex=nx; mousey=ny;
}

// ======================================================= void MouseRect::RightDown(int nx, int ny)
//
void MouseRect::RightDown(int nx, int ny)
{
	if((left==0) && (right==0)) 
	{ 
		ChangeTo(x=clickx=nx,y=clicky=ny,0,0); 
		right=1; 
	}
	else right=-1;
	
	mousex=nx; mousey=ny;
}

// ========================================================= void MouseRect::RightUp(int nx, int ny)
//
void MouseRect::RightUp(int nx, int ny)
{
	if(right==1) 
	{
		ChangeTo(clickx,clicky,nx-clickx,ny-clicky);
		right=0;
	}
	else right=0;
	
	mousex=nx; mousey=ny;
}

// ========================================================== void MouseRect::Motion(int nx, int ny)
//
void MouseRect::Motion(int nx, int ny)
{
	if((left>0) || (right>0)) 
		ChangeTo(clickx,clicky,nx-clickx,ny-clicky);

	mousex=nx; mousey=ny;
}

// =================================================================== int MouseRect::SetMode(int m)
//
int MouseRect::SetMode(int m)
{
	int oldmode=mode;
	mode = m;
	return oldmode;
};

// ============================================================== int MouseRect::GetMode(void) const
//
int MouseRect::GetMode(void) const { return mode; }

// ========================================================== int MouseRect::GetMode(int mask) const
//
int MouseRect::GetMode(int mask) const { return mode & mask; }

// =============================================================== int MouseRect::MouseX(void) const
//
int MouseRect::MouseX(void) const { return mousex; }

// =============================================================== int MouseRect::MouseY(void) const
//
int MouseRect::MouseY(void) const { return mousey; }

// =============================================================== int MouseRect::ClickX(void) const
//
int MouseRect::ClickX(void) const { return clickx; }

// =============================================================== int MouseRect::ClickY(void) const
//
int MouseRect::ClickY(void) const { return clicky; }

// ================================================================= int MouseRect::Left(void) const
//
int MouseRect::Left(void) const { return left; }
// ================================================================ int MouseRect::Right(void) const
//
int MouseRect::Right(void) const { return right; }

// ======================================================== void MouseRect::Draw(int mode = 0) const
//
void MouseRect::Draw(int) const
{
	if((mode & MM_SUBMODE) == 0) // nie rysujemy, gdy nic sie nie dzieje
		return;
		
	glPushAttrib(GL_LINE_BIT|GL_ENABLE_BIT);
		switch(mode & MM_MAINMODE)
		{
			case MM_EDIT: glEnable(GL_LINE_STIPPLE); break;
			default:
				glDisable(GL_LINE_STIPPLE);
		}
		glBegin(GL_LINE_LOOP);   
			glVertex2f(x,y);                                                                      
			glVertex2f(x+width,y);
			glVertex2f(x+width,y+height);
			glVertex2f(x,y+height);
		glEnd();
	glPopAttrib();
}
