#ifdef WIN32
#include <windows.h>
#endif
#include <iostream.h>
#include <assert.h>
#include "Rect.h"

#undef DEBUG

// ================================================================================ Rect::Rect(void)
//
Rect::Rect(void) { x=y=width=height=selected=0; }

// ======================================================================= Rect::Rect(const Rect& r)
//
Rect::Rect(const Rect& r) 
{ 
	#ifdef DEBUG
		cerr<<"<"<<this<<"> Rect::Rect(Rect&r) [x,y,width,height,selected]=["<<r.x<<","<<r.y<<","<<r.width<<","<<r.height<<","<<selected<<"]"<<endl;
	#endif
	// FIXME: sprawdzac ujemne?
	x=r.x; 
	y=r.y; 
	width=r.width; 
	height=r.height; 
	selected=r.selected;
}

// ====================================================== Rect::Rect(int nx, int ny, int nw, int nh)
//
Rect::Rect(int nx, int ny, int nw, int nh) 
{ 
	if(nw<0) x = nx - (width=-nw);
	else { width=nw; x=nx; }
	
  if(nh<0) y = (ny - (height=-nh)); 
	else { height=nh; y=ny; }
	
	#ifdef DEBUG
		cerr<<"<"<<this<<"> (done) Rect::Rect("<<nx<<","<<ny<<","<<nw<<","<<nh<<") [x,y,width,height]=["<<x<<","<<y<<","<<width<<","<<height<<"]"<<endl;
	#endif
}

// =============================================================================== Rect::~Rect(void)
//
Rect::~Rect(void) { }

// ================================================================= void Rect::Move(int nx, int ny)
//
void Rect::Move(int nx, int ny) { x+=nx; y+=ny; }

// ================================================================ void Rect::MoveTo(const Rect& r)
//
void Rect::MoveTo(const Rect& r) { x=r.x; y=r.y; }

// =============================================================== void Rect::MoveTo(int nx, int ny)
//
void Rect::MoveTo(int nx, int ny) { x=nx; y=ny; }

// =============================================================== void Rect::Resize(int nw, int nh)
//
void Rect::Resize(int nw, int nh) 
{ 
	if((width+=nw)<0) x -= (width=-width);
	if((height+=nh)<0) y -= (height=-height);
}

// ============================================================== void Rect::ResizeTo(const Rect& r)
//
void Rect::ResizeTo(int nw, int nh) 
{ 
	if((width=nw)<0) x -= (width=-width);
	if((height=nh)<0) y -= (height=-height);
	#ifdef DEBUG
		cerr<<"<"<<this<<"> (done) Rect::ResizeTo("<<nw<<","<<nh<<") [x,y,width,height]=["<<x<<","<<y<<","<<width<<","<<height<<"]"<<endl;
	#endif

}

// ============================================================== void Rect::ResizeTo(const Rect& r)
//
void Rect::ResizeTo(const Rect& r) 
{ 
	width=r.width; 
	height=r.height; 
}

// =============================================== void Rect::Change(int nx, int ny, int nw, int nh)
//
void Rect::Change(int nx, int ny, int nw, int nh) 
{ 
	#ifdef DEBUG
		cerr<<"<"<<this<<"> Rect::Change("<<x<<","<<y<<","<<nw<<","<<nh<<") "<<endl;
	#endif

	x+=nx;
	y+=ny; 
	if((width=nw)<0) x -= (width=-width);
	if((height=nh)<0) y -= (height=-height);
}

// ============================================= void Rect::ChangeTo(int nx, int ny, int nw, int nh)
//
void Rect::ChangeTo(int nx, int ny, int nw, int nh) 
{ 
	#ifdef DEBUG
		cerr<<"<"<<this<<"> Rect::ChangeTo("<<x<<","<<y<<","<<nw<<","<<nh<<") "<<endl;
	#endif

	if(nw<0) x = (nx - (width=-nw));
	else { width=nw; x=nx; }

  if(nh<0) y = (ny - (height=-nh));
	else { height=nh; y=ny; }
}

// ============================================================== void Rect::ChangeTo(const Rect& r)
//
void Rect::ChangeTo(const Rect& r) 
{ 
	x=r.x; 
	y=r.y; 
	width=r.width; 
	height=r.height; 
}

// ================================================== void Rect::Grow(int nx,int ny, int nw, int nh)
//
void Rect::Grow(int nx,int ny, int nw, int nh)
{
	if((x==0) && (y==0) && (width==0) && (height==0))
		ChangeTo(nx,ny,nw,nh);
	else
	{
		if(nx<x) { width+=(x-nx); x=nx; }
		if(ny<y) { height+=(y-ny); y=ny; }
		if(x+width<nx+nw) width+=(nx+nw-x-width);
		if(y+height<ny+nh) height+=(ny+nh-y-height);
	}
}

// ================================================================== void Rect::Grow(const Rect& r)
//
void Rect::Grow(const Rect& r) { Grow(r.x,r.y,r.width,r.height); }

// ========================================================================= void Rect::Select(void)
//
void Rect::Select(void) { selected = 1; }

// ======================================================================= void Rect::Deselect(void)
//
void Rect::Deselect(void) { selected = 0; }

// ====================================================================== void Rect::InvSelect(void)
//
void Rect::InvSelect(void) { selected = !selected; }

// ================================================================== int Rect::Selected(void) const
//
int Rect::Selected(void) const { return selected; }

// ============================================================ int Rect::InMe(int nx, int ny) const
//
int Rect::InMe(int nx, int ny) const
{
	assert(width>=0);
	assert(height>=0);

	int inx = (x<=nx) && (nx<=x+width); 
	int iny = (y<=ny) && (ny<=y+height);

	#ifdef DEBUG
		cerr<<"<"<<this<<"> (done) "<<(inx && iny)<<" Rect::InMe("<<nx<<","<<ny<<") [x,y,width,height]=["<<x<<","<<y<<","<<width<<","<<height<<"]"<<endl;
	#endif

	return inx && iny;
}

// ============================================================= int Rect::InMe(const Rect& r) const
//
int Rect::InMe(const Rect& r) const { return InMe(r.x,r.y) && InMe(r.x+r.width,r.y+r.height); }

// ========================================================== int Rect::OnEgde(int nx, int ny) const
//
int Rect::OnEgde(int nx, int ny) const 
{
	// TODO: sprawdzenie ujemnych width/height
	int onedge = EDGE_NONE;
	if(InMe(nx,ny))
	{
		onedge |= (x==nx) ? EDGE_RIGHT : ((x==nx+width) ? EDGE_LEFT : EDGE_NONE);
		onedge |= (y==ny) ? EDGE_UP : ((y==ny+height) ? EDGE_DOWN : EDGE_NONE);
	}

	return onedge;
}

// ========================================================================= int Rect::X(void) const
//
int Rect::X(void) const { return x; }

// ========================================================================= int Rect::Y(void) const
//
int Rect::Y(void) const { return y; }

// ===================================================================== int Rect::Width(void) const
//
int Rect::Width(void) const { return width; }

// ==================================================================== int Rect::Height(void) const
//
int Rect::Height(void) const { return height; }

// ========================================================================= Rect Rect::Center(void)
//
Rect Rect::Center(void) { return Rect(x+int(width/2), y+int(height/2),0,0); }

// ============================================================= void Rect::Draw(int mode = 0) const
//
void Rect::Draw(int) const { }
